@@ -7,15 +7,13 @@ import logging
 import copy
 import time
 from datetime import datetime
-
-
+import json
 from urllib.parse import urljoin
 
-
 import requests
-import json
 import yaml
 
+
 start_template = {
     "src": "",
     "name": "",
@@ -47,22 +45,15 @@ def get_graphs(connection) :
     return data
 
 
-def start_graph(connection,pipeline,pipeline_config) :
-
-    params = copy.deepcopy(start_template)
+def start_graph(connection,config) :
 
     restapi = "/runtime/graphs"
     url = connection['url'] + restapi
-    logging.debug(f'Start graph: {pipeline}')
     headers = {'X-Requested-With': 'XMLHttpRequest'}
-    params['src'] = pipeline
-    cname = '-'.join(pipeline_config.values())
-    params['name'] = 'Batch-'+cname
-    params['configurationSubstitutions'] = pipeline_config
-    r = requests.post(url, headers=headers, auth=connection['auth'], data = json.dumps(params))
+    r = requests.post(url, headers=headers, auth=connection['auth'], data = json.dumps(config))
 
     if r.status_code != 200:
-        logging.error(f"Pipeline could not be started: {pipeline} - {r.status_code}\n{r.text}")
+        logging.error(f"Pipeline could not be started: {config['src']} - {r.status_code}\n{r.text}")
         return None
 
     r = json.loads(r.text)
@@ -71,67 +62,96 @@ def start_graph(connection,pipeline,pipeline_config) :
             'submitted': r['submitted'],'started': r['started'],'stopped': r['stopped']}
 
 
-def start_batch(connection,pipelines,max_procs = 2,sleep_time = 2 ) :
+
+def start_batch(connection, batch, max_procs = 2) :
 
     processing = True
     num_procs = 0
-    batch_index = 0
+    index = 0
     procs =dict()
+    auths = {connection['auth']}
     while(processing) :
         # If all pipelines have been processed stop process
-        if batch_index == len(pipelines):
+        if index == len(batch):
             processing = False
             break
         # If there available resources start new pipeline
         if num_procs < max_procs :
-            rec = start_graph(connection,pipelines[batch_index]['pipeline'],pipelines[batch_index]['configuration'])
+            if 'sleep_time' in batch[index]:
+                logging.info(f"Sleep for {batch[index]['sleep_time']}s")
+                time.sleep(batch[index]['sleep_time'])
+                index +=1
+                continue
+
+            conn = connection.copy()
+            if "user" in batch[index] :
+                user_auth = (batch[index]['tenant']+'\\'+ batch[index]['user'],batch[index]['password'])
+                conn['auth'] = user_auth
+                auths.add(user_auth)
+
+            rec = start_graph(conn, batch[index])
             if rec  :
-                logging.info(f"{batch_index} - Pipeline started: {rec['name']}")
-                rec['batch_index'] = batch_index
+                logging.info(f"{index} - Pipeline started: {rec['name']}")
+                rec['batch_index'] = index
                 procs[rec['handle']] = rec
                 num_procs +=1
-            batch_index +=1  # in case of failing start of pipeline - it is skipped
+            index +=1  # in case of failing start of pipeline - it is skipped
+            time.sleep(5)
 
         # Check runtime graphs for changes, update procs and free up proc resources
-        runtime_graphs = get_graphs(connection)
-        for h,rg in runtime_graphs.items():
-            # filter runtime graphs
-            if (not h in procs.keys()) or (procs[h]['status'] in ['dead','completed']) :
+        runtime_graphs = list()
+        for auth in auths :
+            conn = {'url': connection['url'],'auth': auth}
+            runtime_graphs.append(get_graphs(conn))
+
+        for runtime_graph in runtime_graphs:
+            if runtime_graph == None :
+                logging.error(f'Runtime graph is None in {runtime_graphs}')
                 continue
-            # update procs
-            logging.info(f"Check for update: {h}")
-            procs[h]['status'] = rg['status']
-            procs[h]['submitted'] = rg['submitted']
-            procs[h]['started'] = rg['started']
-            procs[h]['stopped'] = rg['stopped']
+            for h,rg in runtime_graph.items():
+                # filter runtime graphs
+                if (not h in procs.keys()) or (procs[h]['status'] in ['dead','completed']) :
+                    continue
+                # update procs
+                #logging.debug(f"Check for update: {h}")
+                procs[h]['status'] = rg['status']
+                procs[h]['submitted'] = rg['submitted']
+                procs[h]['started'] = rg['started']
+                procs[h]['stopped'] = rg['stopped']
 
-            if rg['status'] in ['dead','completed'] :
-                num_procs -=1
-                logging.info(f"Pipeline stopped: {rg['name']} -  {rg['status']}")
-                logging.info(f"New proc resource available: {num_procs}/{max_procs}")
+                if rg['status'] in ['dead','completed'] :
+                    num_procs -=1
+                    logging.info(f"Pipeline stopped: {rg['name']} -  {rg['status']}")
+                    logging.info(f"New proc resource available: {num_procs}/{max_procs}")
 
-        time.sleep(sleep_time)
 
     # Wait until all processes are complete
+
     running = True
     while(running) :
-        runtime_graphs = get_graphs(connection)
+
         running = False
-        for h,rg in runtime_graphs.items():
-            # filter runtime graphs
-            if (not h in procs.keys()) or (procs[h]['status'] in ['dead','completed']) :
-                continue
-            # update procs
-            logging.info(f"Check for update: {h}")
-            procs[h]['status'] = rg['status']
-            procs[h]['submitted'] = rg['submitted']
-            procs[h]['started'] = rg['started']
-            procs[h]['stopped'] = rg['stopped']
+        runtime_graphs = list()
+        for auth in auths :
+            conn = {'url': connection['url'],'auth': auth}
+            runtime_graphs.append(get_graphs(conn))
 
-            if not rg['status'] in ['dead','completed']:
-                running = True
+        for runtime_graph in runtime_graphs:
+            for h,rg in runtime_graph.items():
+                # filter runtime graphs
+                if (not h in procs.keys()) or (procs[h]['status'] in ['dead','completed']) :
+                    continue
+                # update procs
+                #logging.info(f"Check for update: {h}")
+                procs[h]['status'] = rg['status']
+                procs[h]['submitted'] = rg['submitted']
+                procs[h]['started'] = rg['started']
+                procs[h]['stopped'] = rg['stopped']
 
-        time.sleep(sleep_time)
+                if not rg['status'] in ['dead','completed']:
+                    running = True
+
+    logging.info("Batch process ended.")
 
     return procs
 
@@ -146,9 +166,6 @@ if __name__ == '__main__':
     conn = {'url': urljoin(params['URL'] , '/app/pipeline-modeler/service'),
             'auth': (params['TENANT'] + '\\' + params['USER'], params['PWD'])}
 
-
-
-
     pipelines1 = [{ 'pipeline':'utils.conf_datagen', 'configuration':{'name':'extstart-1'}}]
     pipelines2 = [{ 'pipeline':'utils.conf_datagen', 'configuration':{'name':'BS-1'}},
                   { 'pipeline':'utils.conf_datagen', 'configuration':{'name':'BS2-2'}}]
@@ -158,6 +175,6 @@ if __name__ == '__main__':
                   { 'pipeline':'utils.conf_datagen', 'configuration':{'name':'BS2-4'}}]
 
 
-    procs = start_batch(conn,pipelines=pipelines4)
+    procs = start_batch(conn, batch=pipelines4)
 
     print(json.dumps(procs,indent=4))
\ No newline at end of file
